
//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
	Texture   = (Tex);          //apply a texture to the sampler
	MipFilter = NONE;         //sampler states
	MinFilter = NONE;
	MagFilter = NONE;
};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //declare output struct
    vs2ps Out;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

float2 PixelSize;
float rot;
float gruen;
float blau;
//horizontal edge detection


//horizontal and vertical detection for improved result
float4 PS(vs2ps In): COLOR
{
	//the texture coordinate offset with vertical coordinate set to 0
	float2 off = float2(PixelSize.x, 0);
	
	//sample the left and the right neighbouring pixels
	float4 left = tex2D(Samp, In.TexCd - off);
	float4 right = tex2D(Samp, In.TexCd + off);
	
	//the texture coordinate offset with horizonal coordinate set to 0
	off = float2(0, PixelSize.y);
	//sample the upper and the lower neighbouring pixels
	float4 upper = tex2D(Samp, In.TexCd - off);
	float4 lower = tex2D(Samp, In.TexCd + off);
	
	off = float2(PixelSize.x, PixelSize.y);
	float4 lowerright = tex2D(Samp, In.TexCd + off);
	float4 upperleft = tex2D(Samp, In.TexCd - off);
	
	off = float2(PixelSize.x, -PixelSize.y);
	float4 upperright = tex2D(Samp, In.TexCd + off);
	float4 lowerleft = tex2D(Samp, In.TexCd - off);
	
	float4 center = tex2D(Samp, In.TexCd);
	
	
	float oddX = (In.TexCd.x/PixelSize.x)/2;
	oddX = frac(oddX);
	oddX = round(oddX);
	
	float oddY = (In.TexCd.y/PixelSize.y)/2;
	oddY = frac(oddY);
	oddY = round(oddY);
	
	// roter Pixel
	if (oddX < 0.1 && oddY < 0.1)
	{
		return float4 (center.r*rot,(upper.g+lower.g+left.g+right.g)/4*gruen,(upperleft.b+lowerleft.b+upperright.b+lowerright.b)/4*blau,1);
		//return float4(1,1,0,1);
	}
	
	//grüner Pixel in der roten reihe
	if (oddX > 0.1 && oddY < 0.1)
	{
		return float4((left.r+right.r)/2*rot,center.g*gruen,(upper.b+lower.b)/2*blau,1);
	}
	
	//blauer Pixel
	if (oddX > 0.1 && oddY > 0.1)
	{
		return float4((upperleft.r+lowerleft.r+upperright.r+lowerright.r)/4*rot,(upper.g+lower.g+left.g+right.g)/4*gruen,center.b*blau,1);
		//return float4(0,0,1,1);
	}
	
	//grüner Pixel in der blauen reihe
	if (oddX < 0.1 && oddY > 0.1)
	{
		return float4((upper.r+lower.r)/2*rot,center.g*gruen,(left.b+right.b)/2*blau,1);
	}
	
	return float4(0,0,0,1);
	
	
}



technique TEdgeDetect
{
	pass P0
	{
		VertexShader = compile vs_1_1 VS();
		PixelShader  = compile ps_3_0 PS();
	}
}
